import logging
from haystack import Finder
from haystack.preprocessor.cleaning import clean_wiki_text
from haystack.preprocessor.utils import tika_convert_files_to_dicts, fetch_archive_from_http
from haystack.reader.farm import FARMReader
from haystack.reader.transformers import TransformersReader
from haystack.utils import print_answers

from haystack.document_store.elasticsearch import ElasticsearchDocumentStore

import sys

args = [arg for arg in sys.argv[1:] if not arg.startswith("-")]

iname = args[0]

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

from elasticsearch import Elasticsearch
es = Elasticsearch([{'host':'127.0.0.1','port':9200}])

print("ES up !")
esCreated = False 
if es.indices.exists(index=iname):
  logger.info("Index "+iname+ " exists. Deleting it")
  es.indices.delete(index=iname,ignore=[400, 404])

if not es.indices.exists(index=iname):
   logger.info("Creating new index "+iname)
# Convert files to dicts
#   doc_dir = "/gstore/home/madabhuc/PRESTODocuments"
   document_store = ElasticsearchDocumentStore(index=iname,embedding_dim=128,timeout=40)
   doc_dir = "/gstore/home/madabhuc/eval"
#  doc_dir = "/gstore/home/madabhuc/trial"
   dicts = tika_convert_files_to_dicts(dir_path=doc_dir, clean_func=clean_wiki_text, split_paragraphs=True)
#  dicts = tika_convert_files_to_dicts(dir_path=doc_dir, clean_func=clean_wiki_text)
# Now, let's write the dicts containing documents to our DB.
   document_store.write_documents(dicts,index=iname)
   logger.info(str(len(dicts)) + " Documents stored in ES index "+iname)
   esCreated = True 
else:
   document_store = ElasticsearchDocumentStore(index=iname,embedding_dim=128,timeout=40)

from haystack.retriever.dense import DensePassageRetriever
retriever = DensePassageRetriever(document_store=document_store,
                                  passage_embedding_model="/gstore/home/madabhuc/hayEnv/runs/document_checkpoint_REALM",
                                  query_embedding_model="/gstore/home/madabhuc/hayEnv/runs/question_checkpoint_REALM",
                                  max_seq_len_query=64,
                                  max_seq_len_passage=256,
                                  batch_size=16,
                                  use_gpu=True,
                                  embed_title=True,
                                  use_fast_tokenizers=True)
logger.info("Retriever created")
if esCreated :
   document_store.update_embeddings(retriever)
   logger.info("Updated embeddings")

#reader = FARMReader(model_name_or_path="deepset/roberta-base-squad2", use_gpu=True)

#print("Reader created")

finder = Finder(reader=None, retriever=retriever)

logger.info("Finder created")

q="how long is the treatment period"
logger.info("Question: "+q)
prediction = finder.get_answers_via_similar_questions(question=q, top_k_retriever=10)
#print_answers(prediction, details="minimal") #medium prints out score too
print_answers(prediction, details="medium")

q="what is the rationale for the drug dose"

print("Question: ",q)
prediction = finder.get_answers_via_similar_questions(question=q, top_k_retriever=10)
print_answers(prediction, details="medium")

q="what is the rationale for the drug dosage"
print("Question: ",q)
prediction = finder.get_answers_via_similar_questions(question=q, top_k_retriever=10)
print_answers(prediction, details="medium")

